%
% fdm_diffusion
%
% compute the diffusion term
%

Nx = data.Nx;
Ny = data.Ny;

Indx_inv = data.Indx_inv;
Indx     = data.Indx;

nodes_int = data.nodes_int;

% calculo de la contribucion del operador del interior
for k=1:length(nodes_int)
    nodes = nodes_int(k);
    indx=Indx_inv(nodes,1);
    jndx=Indx_inv(nodes,2);

    dx_fwd = (xnod(Indx(indx+1,jndx),1)-xnod(Indx(indx,jndx),1));
    dx_bkw = (xnod(Indx(indx,jndx),1)-xnod(Indx(indx-1,jndx),1));
    dx_cen = (xnod(Indx(indx+1,jndx),1)-xnod(Indx(indx-1,jndx),1));

    dy_fwd = (xnod(Indx(indx,jndx+1),2)-xnod(Indx(indx,jndx),2));
    dy_bkw = (xnod(Indx(indx,jndx),2)-xnod(Indx(indx,jndx-1),2));
    dy_cen = (xnod(Indx(indx,jndx+1),2)-xnod(Indx(indx,jndx-1),2));

    dx_half = 0.5*(dx_fwd+dx_bkw);
    dy_half = 0.5*(dy_fwd+dy_bkw);

    kappa_x_plus = (kappa(Indx(indx+1,jndx),1)+kappa(Indx(indx,jndx),1))/2;
    kappa_x_minus = (kappa(Indx(indx,jndx),1)+kappa(Indx(indx-1,jndx),1))/2;
    kappa_y_plus = (kappa(Indx(indx,jndx+1),1)+kappa(Indx(indx,jndx),1))/2;
    kappa_y_minus = (kappa(Indx(indx,jndx),1)+kappa(Indx(indx,jndx-1),1))/2;

    grad_phi_x_plus = (state(Indx(indx+1,jndx),1)-state(Indx(indx,jndx),1)) ./ dx_fwd ;
    grad_phi_x_minus = (state(Indx(indx,jndx),1)-state(Indx(indx-1,jndx),1)) ./ dx_bkw ;

    grad_phi_y_plus = (state(Indx(indx,jndx+1),1)-state(Indx(indx,jndx),1)) ./ dy_fwd ;
    grad_phi_y_minus = (state(Indx(indx,jndx),1)-state(Indx(indx,jndx-1),1)) ./ dy_bkw ;

    flux_d_x_plus  = kappa_x_plus.*grad_phi_x_plus;
    flux_d_x_minus = kappa_x_minus.*grad_phi_x_minus;
    flux_d_y_plus  = kappa_y_plus.*grad_phi_y_plus;
    flux_d_y_minus = kappa_y_minus.*grad_phi_y_minus;

    R(Indx(indx,jndx),1) = (flux_d_x_plus - flux_d_x_minus)./dx_half + ...
        (flux_d_y_plus - flux_d_y_minus)./dy_half + data.source(nodes) - ...
        data.reac(nodes)*(state(Indx(indx,jndx),1)) - ...
        data.param.rho*data.param.Cp/data.param.dt*(state(Indx(indx,jndx),1) - state_old(Indx(indx,jndx),1)) ;

% add advection terms
    R(Indx(indx,jndx),1) = R(Indx(indx,jndx),1) - ( ... 
                           data.veloc(nodes,1)./(xnod(Indx(indx+1,jndx),1)-xnod(Indx(indx-1,jndx),1)) .* ...        
                           (state(Indx(indx+1,jndx),1)-state(Indx(indx-1,jndx),1)) + ...
                           data.veloc(nodes,2)./(xnod(Indx(indx,jndx+1),2)-xnod(Indx(indx,jndx-1),2)) .* ...
                           (state(Indx(indx,jndx+1),1)-state(Indx(indx,jndx-1),1)) );

    if strcmp(job_flag,'all')

        K0 = ( kappa_x_plus.*(-1 ./ dx_fwd) - kappa_x_minus.*(1 ./ dx_bkw) )./ dx_half ;
        K0 = K0 + (kappa_y_plus.*(-1 ./ dy_fwd) - kappa_y_minus.*(1 ./ dy_bkw) )./ dy_half ;
        K0 = K0 - data.reac(nodes);
        K0 = K0 - data.param.rho*data.param.Cp/data.param.dt;
       
        KE = kappa_x_plus ./ dx_fwd ./ dx_half;
        KE = KE - data.veloc(nodes,1)./(xnod(Indx(indx+1,jndx),1)-xnod(Indx(indx-1,jndx),1));
                             
        KW = kappa_x_minus ./ dx_bkw ./ dx_half;
        KW = KW + data.veloc(nodes,1)./(xnod(Indx(indx+1,jndx),1)-xnod(Indx(indx-1,jndx),1));
        
        KN = kappa_y_plus ./ dy_fwd ./ dy_half;
        KN = KN - data.veloc(nodes,2)./(xnod(Indx(indx,jndx+1),2)-xnod(Indx(indx,jndx-1),2));
        
        KS = kappa_y_minus ./ dy_bkw ./ dy_half;
        KS = KS + data.veloc(nodes,2)./(xnod(Indx(indx,jndx+1),2)-xnod(Indx(indx,jndx-1),2));

        rows = Indx(indx,jndx); cols = rows; coef = K0;
        rowg = [rowg;rows]; colg = [colg;cols]; coeg = [coeg;coef];

        cols = Indx(indx-1,jndx); coef = KW;
        rowg = [rowg;rows]; colg = [colg;cols]; coeg = [coeg;coef];

        cols = Indx(indx+1,jndx); coef = KE;
        rowg = [rowg;rows]; colg = [colg;cols]; coeg = [coeg;coef];

        cols = Indx(indx,jndx+1); coef = KN;
        rowg = [rowg;rows]; colg = [colg;cols]; coeg = [coeg;coef];

        cols = Indx(indx,jndx-1); coef = KS;
        rowg = [rowg;rows]; colg = [colg;cols]; coeg = [coeg;coef];

    end

end

bc.Neumann = data.bc.Neumann;

% calculo de la contribucion de la condicion de contorno Neumann
if isempty(bc.Neumann.nodes)==0,
    nodes_bc = bc.Neumann.nodes;
    flujo_bc = bc.Neumann.value;
    normal_bc = bc.Neumann.normal;
    for k=1:length(nodes_bc)
        nodes = nodes_bc(k);
        indx=Indx_inv(nodes,1);
        jndx=Indx_inv(nodes,2);

        indx_ext = indx+sign(normal_bc(k,1));
        jndx_ext = jndx+sign(normal_bc(k,2));

        indx_int = indx-sign(normal_bc(k,1));
        jndx_int = jndx-sign(normal_bc(k,2));

        dx = ( xnod(Indx(indx_ext,jndx_ext),1) - xnod(Indx(indx_int,jndx_int),1));
        dy = ( xnod(Indx(indx_ext,jndx_ext),2) - xnod(Indx(indx_int,jndx_int),2));
        
        dl = sqrt(dx.^2+dy.^2);
        R(Indx(indx_ext,jndx_ext),1) = R(Indx(indx_ext,jndx_ext),1) + kappa(Indx(indx,jndx),1) .* ...
            ( state(Indx(indx_ext,jndx_ext),1) - state(Indx(indx_int,jndx_int),1)) ./ dl + ...
            flujo_bc(k) ;
        
        if strcmp(job_flag,'all')

            KE =  kappa(Indx(indx,jndx),1) ./ dl * (sign(normal_bc(k,1)));
            KW = -kappa(Indx(indx,jndx),1) ./ dl * (sign(normal_bc(k,1)));

            KN =  kappa(Indx(indx,jndx),1) ./ dl * (sign(normal_bc(k,2)));
            KS = -kappa(Indx(indx,jndx),1) ./ dl * (sign(normal_bc(k,2)));
            
            rows = Indx(indx_ext,jndx); cols = Indx(indx_int,jndx); coef = KW;
            rowg = [rowg;rows]; colg = [colg;cols]; coeg = [coeg;coef];

            rows = Indx(indx_ext,jndx); cols = Indx(indx_ext,jndx); coef = KE;
            rowg = [rowg;rows]; colg = [colg;cols]; coeg = [coeg;coef];

            rows = Indx(indx,jndx_ext); cols = Indx(indx,jndx_ext); coef = KN;
            rowg = [rowg;rows]; colg = [colg;cols]; coeg = [coeg;coef];

            rows = Indx(indx,jndx_ext); cols = Indx(indx,jndx_int); coef = KS;
            rowg = [rowg;rows]; colg = [colg;cols]; coeg = [coeg;coef];

        end

    end
end
